<%
// login.cs
// Copyright (c) 2002 - 2005 Citrix Systems, Inc. All Rights Reserved.
// Web Interface (Build 45083)
%>

<!--#include file="loginView.cs"-->

<script runat="server">

private PageAction loginAuthenticateExplicit(ExplicitAuth expAuth) {
    PageAction result = null; // default to rendering the login page again

    if (Request.HttpMethod != "POST") {
        // Explicit logins can only be performed via an HTTP POST
        return result;
    }

    AuthenticationConfiguration authConfig = getAuthenticationConfiguration();

    // Pull out the fields we have interest in
    string user = "";
    if ( Request.Params[ID_USER] != null ) {
        user = ((string)Request.Params[ID_USER]).Trim();
    }

    string password = "";
    if ( Request.Params[ID_PASSWORD] != null ) {
        password = ((string)Request.Params[ID_PASSWORD]).Trim();
    }

    string domain = "";
    if ( Request.Params[ID_DOMAIN] != null ) {
        domain = ((string)Request.Params[ID_DOMAIN]).Trim();
    }

    string context = "";
    if ( Request.Params[ID_CONTEXT] != null ) {
        context = ((string)Request.Params[ID_CONTEXT]).Trim();
    }

    string passcode = "";
    if( Request.Form[ID_PASSCODE] != null ) {
        passcode = ((string)Request.Form[ID_PASSCODE]).Trim();
    }

    string otc = "";
    if( Request.Form[ID_OTC] != null ) {
        otc = ((string)Request.Form[ID_OTC]).Trim();
    }

    // Make sure none of the credential fields contain control characters
    if( Strings.hasControlChars( user )
        || Strings.hasControlChars( password )
        || Strings.hasControlChars( domain )
        || Strings.hasControlChars( context )
        || Strings.hasControlChars( passcode ) 
        || Strings.hasControlChars( otc ) ) {
        messageCenterControl.setType( MessageCenterControl.MSGTYPE_ERROR );
        messageCenterControl.setKey( "InvalidCredentials" );
    } else if (expAuth is ExplicitNDSAuth) {
        // Defer to special NDS processing code that handles context lookup
        result = loginAuthenticateNDS((ExplicitNDSAuth) expAuth, user, password, passcode, context);
    } else {
        // Not NDS - parse the fields and go do any Two-factor
        ExplicitUDPAuth udpAuth = (ExplicitUDPAuth) expAuth;
        AccessToken credentials = null;

        if (user == null || user.Length == 0) {
            messageCenterControl.setType( MessageCenterControl.MSGTYPE_ERROR );
            messageCenterControl.setKey( "BlankUsername" );
        } else if (user.IndexOf("@") >= 0) {
            // User has entered a UPN
            // Validate there are username and domain parts to the UPN.
            string[] userElems = user.Split(new char[] { '@' }, 2);
            if ( userElems[0] == null || userElems[0].Trim() == "" ) {
                // Avoid cases where user has entered '@domain'
                messageCenterControl.setType( MessageCenterControl.MSGTYPE_ERROR );
                messageCenterControl.setKey( "BlankUsername" );
            } else if ( userElems.Length < 2 || userElems[1] == null || userElems[1].Trim() == "" ) {
                // Avoid cases where user has entered 'user@'
                messageCenterControl.setType( MessageCenterControl.MSGTYPE_ERROR );
                messageCenterControl.setKey( "BlankDomain" );
            } else {
                credentials = new UPNCredentials( user, password );
            }
        } else if (user.IndexOf("\\") >= 0) {
            // User has entered an NT-style domain\user
            string[] userElems = user.Split(new char[] { '\\' }, 2);
            if ( userElems[0] == null || userElems[0].Trim() == "" ) {
                // Avoid cases where user has entered \user
                messageCenterControl.setType( MessageCenterControl.MSGTYPE_ERROR );
                messageCenterControl.setKey( "BlankDomain" );
            } else if ( userElems.Length < 2 || userElems[1] == null || userElems[1].Trim() == "" ) {
                // Avoid cases where user has entered domain\
                messageCenterControl.setType( MessageCenterControl.MSGTYPE_ERROR );
                messageCenterControl.setKey( "BlankUsername" );
            } else {
                credentials = new UserDomainPasswordCredentials( userElems[1], userElems[0], password );
            }
        } else if ( domain.Length != 0 ) {
            // User has entered User/Domain/Password
            credentials = new UserDomainPasswordCredentials( user, domain, password );
        } else {
            // The user has not entered a domain in any form, use the first in the login domain list
            // (if appropriate).
            if ((udpAuth != null) && udpAuth.getDomainFieldHidden() && !udpAuth.getDomains().isEmpty()) {
                // WI is configured to hide the domain field and the
                // user does not specify a domain in the username
                // field. We attempt to extract the first domain in
                // the LoginDomains list as the default domain.
                string d = ( string ) udpAuth.getDomains().get( 0 );
                if (d != null) {
                    // Note: The domain can be empty if explicitly set by the admin.  This allows
                    // Unix-style credentials, with no domain.
                    credentials = new UserDomainPasswordCredentials( user, d, password );
                }
            } else {
                messageCenterControl.setType( MessageCenterControl.MSGTYPE_ERROR );
                messageCenterControl.setKey( "BlankDomain" );
            }
        }

        if (credentials != null && !bIsError()) {
            // Load settings
            bool ssl = false;
            if (ConfigurationSettings.AppSettings["PINsafe_SSL"] != null)
                ssl = ConfigurationSettings.AppSettings["PINsafe_SSL"].ToLower().Equals("true");

            string server = "localhost";
            if (ConfigurationSettings.AppSettings["PINsafe_Server"] != null)
                server = ConfigurationSettings.AppSettings["PINsafe_Server"];

            int port = 8080;
            if (ConfigurationSettings.AppSettings["PINsafe_Port"] != null)
                try {
                    port = Int32.Parse(ConfigurationSettings.AppSettings["PINsafe_Port"]);
                } catch (FormatException e) { }

            string psContext = "pinsafe";
            if (ConfigurationSettings.AppSettings["PINsafe_Context"] != null)
                psContext = ConfigurationSettings.AppSettings["PINsafe_Context"];

            string secret = "";
            if (ConfigurationSettings.AppSettings["PINsafe_Secret"] != null)
                secret = ConfigurationSettings.AppSettings["PINsafe_Secret"];

            // Configure client
            PINsafeClient pc = new PINsafeClient();
            pc.SSL = ssl;
            pc.Server = server;
            pc.Port = port;
            pc.Context = psContext;
            pc.Secret = secret;
          
            // Perform PINsafe authentication
            if (!pc.Login(user, "", otc)) {
                // Set generic authentication error on failure
                messageCenterControl.setType( MessageCenterControl.MSGTYPE_ERROR );
                messageCenterControl.setKey( "InvalidCredentials" );
            }
        }

        if (credentials != null && !bIsError()) {

            System.Collections.Hashtable parameters = new System.Collections.Hashtable();
            parameters["AccessToken"] = credentials;
            parameters["ExplicitAuth"] = expAuth;

            getAuthenticationState(Context).addPageToQueueHead("explicit", parameters);

            // If two-factor authentication is in use, insert it in the sequence
            TwoFactorAuthMethod twoFactorMethod = getTwoFactorAuthMethod();
            if (twoFactorMethod != null) {
                parameters["Passcode"] = passcode;
                getAuthenticationState(Context).addPageToQueueHead(twoFactorMethod.getName().ToLower(), parameters);
            }

            getAuthenticationState(Context).pageCompleted();

            // Move on to the next page - no need to redirect
            result = new PageAction( getAuthenticationState(Context).getCurrentPage(), false );
        }
    }

    return result;
}

private PageAction loginAuthenticateIntegrated() {
    getAuthenticationState(Context).addPageToQueueHead("integrated", null);
    getAuthenticationState(Context).pageCompleted();

    // redirect to integrated page, otherwise we may not impersonate
    return new PageAction( getAuthenticationState(Context).getCurrentPage(), true );
}

private PageAction loginAuthenticateCertificate() {
    if (!Request.IsSecureConnection) {
        // Redirect to HTTPS so the user's cert can be sent.  When we do this, the
        // user's session is lost so we have to indicate to ourselves that the user
        // has already chosen their logon method
        string host    = Request.ServerVariables[SRV_SERVER_NAME];
        string urlPath = Request.ServerVariables[SRV_URL];
        Session.Abandon();
        string url = "https://" + host + urlPath + "?" + QSTR_LOGINTYPE + "=" + AuthMethod.CERTIFICATE;
        return new PageAction( url, true );
    } else {
        getAuthenticationState(Context).addPageToQueueHead("certificate", null);
        getAuthenticationState(Context).pageCompleted();
        // redirect here otherwise certificate page may not impersonate
        return new PageAction( getAuthenticationState(Context).getCurrentPage(), true );
    }
}

private PageAction loginAuthenticateGuest() {
    getAuthenticationState(Context).addPageToQueueHead("anonymous", null);
    getAuthenticationState(Context).pageCompleted();
    return new PageAction( getAuthenticationState(Context).getCurrentPage(), false );
}

/**
 * Interpret and process a login request.
 *
 * This method handles login requests that result from form POST data, or
 * from a GET request.  This method will fall-through when the given data
 * is obviously incorrect (having set appropriate message text in the
 * message center control).
 */
public void loginHandleLoginRequest( string loginType ) {
    PageAction pageAction = null; // default to rendering the login page

    AuthenticationConfiguration authConfig = getAuthenticationConfiguration();
    if (authConfig.isEnabledMethod( loginType )) {
        if (AuthMethod.EXPLICIT == loginType) {
            pageAction = loginAuthenticateExplicit((ExplicitAuth)authConfig.getMethod(AuthMethod.EXPLICIT));
        } else if ((AuthMethod.SINGLE_SIGN_ON == loginType) || (AuthMethod.CERTIFICATE_SINGLE_SIGN_ON == loginType)) {
            pageAction = loginAuthenticateIntegrated();
        } else if (AuthMethod.CERTIFICATE == loginType) {
            pageAction = loginAuthenticateCertificate();
        } else if (AuthMethod.ANONYMOUS == loginType) {
            pageAction = loginAuthenticateGuest();
        }

        // Move the browser on to the next authentication page, if appropriate.
        if (pageAction != null) {
            // Not just rendering the logon page, so record the user's choice
            // of logon method.
            authStoreLogonType( loginType );

            envAdaptor.commitState();
            envAdaptor.destroy();

            // Script execution is always stopped by one of the following methods
            if( pageAction.getUseRedirect() ) {
                Response.Redirect(pageAction.getURL());
            } else {
                Server.Transfer(pageAction.getURL());
            }
        }
    }
}

public void loginProcessPost() {
    // Abort if there is an error to display
    if (bIsError()) {
        return;
    }

    // This is a login request resulting from a POST of form data.
    //
    // First, check what kind of submission this is.
    //
    string formAction = (string)Request.Form[ID_SUBMIT_MODE];
    string languageStr = (string) Request.Form[ID_OPTION_LANGUAGE];

    // This submission is purely to apply the selection of preferred language.
    //
    if ( formAction == MODE_APPLY ) {
        setLoginOptions(true, null, null, null, null, languageStr);
        return;
    }

    // This is a full form submission.
    //
    string loginType = (string)Request.Form[ID_LOGIN_TYPE];
    bool bAdvOptions = equalsIgnoreCase(VAL_TRUE, (string) getNonEncodedCookie(COOKIE_ADV_OPTIONS_OPENED));
    string clientType = (string) Request.Form[ID_CLIENT_TYPE];
    string bandwidth = (string) Request.Form[ID_BANDWIDTH];
    string reconnectAtLogin = (string) Request.Form[ID_RECONNECT_LOGIN_OPTION];

    // Normal submission, store any of the user's login-time preferences
    setLoginOptions(bAdvOptions, loginType, clientType, bandwidth, reconnectAtLogin, languageStr);

    // Handle the actual login request
    loginHandleLoginRequest( loginType );
}

/**
 * Attempt to perform silent authentication if appropriate.
 *
 * If anonymous authentication is the only method allowed, then silent
 * authentication will always be performed.
 *
 * Other implicit login methods, in order of precedence, are as follows:
 *
 * - Smartcard Passthrough
 * - Passthrough
 * - Smartcard
 *
 * If any of the above methods is the only authentication method allowed, then
 * silent authentication will be performed.
 *
 * If they are among several allowed methods, then silent authentication is
 * performed (using the priority order above) unless the user's preferences
 * indicate otherwise.
 *
 * In all cases except anonymous, silent authentication is dependent on
 * Internet Explorer.
 *
 */
private void loginProcessDirectLogin() {

    AuthenticationConfiguration authConfig = getAuthenticationConfiguration();
    string logonMode = null;

    // Silent authentication is the default if user preference is not set
    bool silentFlag = !java.lang.Boolean.FALSE.Equals( userPrefs.getUseSilentAuth() );
    bool oneLoginType = ( loginTypeCount( sClientInfo.isIE() ) == 1 );

    if ( oneLoginType && authConfig.isEnabledMethod( AuthMethod.ANONYMOUS ) ) {
        logonMode = AuthMethod.ANONYMOUS;
    } else if (sClientInfo.isIE()) {
        if ( oneLoginType && authConfig.isEnabledMethod( AuthMethod.CERTIFICATE_SINGLE_SIGN_ON ) ) {
            logonMode = AuthMethod.CERTIFICATE_SINGLE_SIGN_ON;
        } else if ( oneLoginType && authConfig.isEnabledMethod( AuthMethod.SINGLE_SIGN_ON ) ) {
            logonMode = AuthMethod.SINGLE_SIGN_ON;
        } else if ( oneLoginType && authConfig.isEnabledMethod( AuthMethod.CERTIFICATE ) ) {
            logonMode = AuthMethod.CERTIFICATE;
        } else if (silentFlag) {
            // This means that more than one authentication method is available
            if ( authConfig.isEnabledMethod( AuthMethod.CERTIFICATE_SINGLE_SIGN_ON ) ) {
                logonMode = AuthMethod.CERTIFICATE_SINGLE_SIGN_ON;
            } else if ( authConfig.isEnabledMethod( AuthMethod.SINGLE_SIGN_ON ) ) {
                logonMode = AuthMethod.SINGLE_SIGN_ON;
            } else if ( authConfig.isEnabledMethod( AuthMethod.CERTIFICATE ) ) {
                logonMode = AuthMethod.CERTIFICATE;
            }
        }
    }

    if (logonMode != null) {
        // Attempt the silent authentication
        loginHandleLoginRequest( logonMode );
    }
}

// Check to see number of authentication methods for IE and Netscape.
// As Netscape does not support SmartCard and Desktop credential pass-through.
int loginTypeCount( bool isIE ) {
    AuthenticationConfiguration authConfig = getAuthenticationConfiguration();

    int result = 0;

    if ( authConfig.isEnabledMethod( AuthMethod.EXPLICIT ) ) {
        result++;
    }
    if ( authConfig.isEnabledMethod( AuthMethod.ANONYMOUS ) ) {
        result++;
    }

    if ( isIE ) {
        if ( authConfig.isEnabledMethod( AuthMethod.CERTIFICATE ) ) {
            result++;
        }
        if ( authConfig.isEnabledMethod( AuthMethod.CERTIFICATE_SINGLE_SIGN_ON ) ) {
            result++;
        }
        if ( authConfig.isEnabledMethod( AuthMethod.SINGLE_SIGN_ON ) ) {
            result++;
        }
    }

    return result;
}

private string loginGetDefaultLoginType( string lm ) {

    AuthenticationConfiguration authConfig = getAuthenticationConfiguration();

    // If the last logon mode is recorded and it is still permitted, then
    // use it. Otherwise decide on default logon type.
    // First clear associated variables
    String logonMode = "";

    if ((lm != null) && (lm != "") && authConfig.isEnabledMethod( lm ) ) {
        return lm;
    } else {

        // Use default - this is effectively the precedence order
        if ( authConfig.isEnabledMethod( AuthMethod.EXPLICIT ) ) {
            logonMode = AuthMethod.EXPLICIT;
        } else if ( authConfig.isEnabledMethod( AuthMethod.CERTIFICATE ) ) {
            logonMode = AuthMethod.CERTIFICATE;
        } else if ( authConfig.isEnabledMethod( AuthMethod.SINGLE_SIGN_ON ) ) {
            logonMode = AuthMethod.SINGLE_SIGN_ON;
        } else if ( authConfig.isEnabledMethod( AuthMethod.CERTIFICATE_SINGLE_SIGN_ON ) ) {
            logonMode = AuthMethod.CERTIFICATE_SINGLE_SIGN_ON;
        } else if ( authConfig.isEnabledMethod( AuthMethod.ANONYMOUS ) ) {
            logonMode = AuthMethod.ANONYMOUS;
        }
    }
    return logonMode;
}

private void setLoginOptions(bool bAdvOptions, string logonMode, string clientType, string bandwidth, string reconnectAtLogin, string languageStr) {
    // Save any updated logon preferences.
    UserPreferences newUserPrefs = new UserPreferences( userPrefs );

    if (logonMode != null) {
        newUserPrefs.setAuthMethod( logonMode );
    }

    if (bAdvOptions) {
        if (clientType != null) {
            newUserPrefs.setClientType( MPSClientType.fromString( clientType ) );
        }
        if (bandwidth != null) {
            newUserPrefs.setBandwidth( BandwidthProfile.fromString( bandwidth ) );
        }
        if (reconnectAtLogin != null) {
            newUserPrefs.setReconnectAtLoginAction( ReconnectOption.fromString( reconnectAtLogin ) );
        }
        if (languageStr != null) {
            Locale l = Utilities.stringToLocale(languageStr);
            if ( getLanguageManager().isLanguageAvailable(l) ) {
                newUserPrefs.setLocale(l);
            }
        }
    } else {
        // Override the client type in case auto fall back to Java Client
        // has taken effect.
        newUserPrefs.setClientType( getSelectedClient() );
    }

    saveUserPrefs( newUserPrefs, envAdaptor );
    userPrefs = getUserPrefs( envAdaptor );
}

private bool bIsError() {
    return messageCenterControl.getType() != MessageCenterControl.MSGTYPE_NONE;
}

</script>
